#!/usr/bin/perl
 
# Copyright(C) 2005, International Business Machines Corporation.  All
# Rights Reserved.

# Script for samba/ldap configuration
#
# Usage: "config_rhel3_test.pl -c {configuration file}
#                              -s {server ip address}
#                              [-l {logfile} [-?]"
# where:
#      -s = server ip address
#      -l = filename for debug output (optional)
#      -? = show usage
#
# NOTE: For ldapsam backend, if you are behind a firewall you may need
#       to authenticate out before running this script.  You may also
#       need to configure CPAN (run "perl -e shell -MCPAN", answer "no"
#       to the configuration prompt, then "quit").

# ====================
# INITIAL SCRIPT SETUP
# ====================

# Declare modules to use
use Socket;
use Getopt::Std;
use File::Copy;
use Cwd;
use lib "../util";
use test_tools;

# Declare Options hash
my %Options;

# Declare some variables
my $rootdn;
my $rootpw;
my $testconf;
my $fileserver;
my $printserver;
my $domainmember;
my $standalone;

# Set up options (See usage)
my $ok = getopts('c:s:l:?', \%Options);
if ( (!$ok) || ($Options{'?'}) ) {
    usage();
    exit(1);
}
$testconf = $Options{c};
if (!$testconf) {
    log_message("Please specify -c <configuration file>");
    usage();
    exit(1);
}
$server = $Options{s};
if (!$server) {
    log_message("Please specify -s <server ip address>");
    usage();
    exit(1);
}
$logfile = $Options{l};
if (!$logfile) {
    $logfile = "/dev/null";
}
open_logfile;

# Administrative user
my $admin = "root";

# Get the configuration options
read_test_conf($testconf);
if ($pdc eq "yes") {
    $domaincontroller = "yes";
} elsif ($member eq "rpc" || $member eq "ads") {
    $domainmember = "yes";
} else {
    $standalone = "yes";
}
if (@fileshares) {
    $fileserver = "yes";
}
if (@printshares) {
    $printserver = "yes";
}
$rootpw = $test_passwd;

# ======================================
# CONFIGURE NSS, LDAP, SAMBA, AND IDEALX
# ======================================

# Save original copy of smb.conf
system_logged("cp /etc/samba/smb.conf /etc/samba/smb.conf.orig");

# Set up samba workgroup
configure_samba_workgroup($domain);

# Configure ldap for ldapsam backend
if (($domaincontroller || $standalone) && $backend eq "ldapsam") {
    # Make copies of the configuration files
    system_logged("cp /etc/ldap.conf /etc/ldap.conf.orig");
    system_logged("cp /etc/openldap/ldap.conf /etc/openldap/ldap.conf.orig");
    system_logged("cp /etc/openldap/slapd.conf /etc/openldap/slapd.conf.orig");
    system_logged("cp /etc/nsswitch.conf /etc/nsswitch.conf.orig");

    # Construct the ldap rootdn
    $rootdn = get_rootdn($suffix);

    # Get the hostname
    my $host = get_host();

    # Configure nss for ldap
    configure_nss($host);

    # Configure ldap server
    configure_ldap_server($host, $suffix, $rootdn, $rootpw);

    # Add the samba schema to the ldap configuration file
    addschema();

    # Configure smbldap tools
    configure_smbldap_tools($suffix, $rootdn, $rootpw, $domain);

    # Add basic group info to ldap database
    system_logged("$idealx/smbldap-populate <<EOF
$rootpw
$rootpw
EOF");

    # Set up ldap client
    configure_ldap_client($server, $suffix);

    # If ldapsam:trusted=yes get rid of the ldap root and nobody users
    # and add ldap groups to the unix root and nobody users
    if ($ldapsamtrusted eq "yes") {
        system_logged("$idealx/smbldap-userdel root");
        system_logged("$idealx/smbldap-userdel nobody");
        system_logged("usermod -g 512 root");
        system_logged("usermod -g 514 nobody");
    }
}

# ==============================================
# SET UP SAMBA FOR SERVER, CLIENT, OR STANDALONE
# ==============================================

# Handle server option
if ($domaincontroller) {
    setup_groups("domadmins", "Domain Admins", "domusers", "Domain Users");
    if ($guest) {
        create_guest;
    }
    setup_samba_auth("pdc", $rootpw, $suffix, $rootdn);
} elsif ($standalone) {
    setup_groups("sambaadmins", "Administrators", "sambausers", "Users");
    if ($guest) {
        create_guest;
    }
    setup_samba_auth("standalone", $rootpw, $suffix, $rootdn);
} elsif ($domainmember) {
    setup_member($rootpw);
}

# Create users
if (@users) {
    create_users;
}

# Set up shares
if ($fileserver) {
    create_file_shares($rootpw);
}
if ($printserver) {
    create_print_shares;
}

# End of script
close_logfile;

# ===============================================
# DEFINE DISTRIBUTION-SPECIFIC SCRIPT SUBROUTINES
# ===============================================

# Function to add the samba schema
sub addschema {
    my $file = '/etc/openldap/slapd.conf';
    my $kerbschema = 'kerberosobject.schema';
    my $sambaschema = `ls /usr/share/doc/samba*/LDAP/samba.schema`;

    # Open the file
    if (!open(FILE, "<$file")) {
        log_message("Could not open $file");
        exit(1);
    }

    # Read and close the file
    my @lines = <FILE>;
    close(FILE);

    # Check if samba schema already included
    my $sambaindex = 0;
    my $lastindex = $#lines;
    foreach (@lines) {
        last if ($_ =~ $sambaschema);
        $sambaindex++;
    }
    if ($sambaindex < $lastindex) {
        log_message("Samba schema already in conf file");
    } else {
        # Find the line where we want to insert the samba schema
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $kerbschema);
            $index++;
        }

        # splice inserts before the index
        $index++;

        # Insert a line into the array
        splice(@lines,$index,0,"include         $sambaschema\n");

        # Rewrite the file
        if (!open(FILE, ">$file")) {
            log_message("Could not open $file for writing");
            exit(1);
        }
        print FILE @lines;
        close(FILE);
    }

    # Enable ldap
    log_message("Enabling ldap services");
    system_logged("chkconfig ldap on");

    # Start ldap
    log_message("Starting ldap");
    system_logged("chmod a+r $file");
    system_logged("/etc/init.d/ldap restart");
}

# Function to configure samba
sub configure_samba_workgroup {
    my $wg = shift;

    # Set the workgroup
    my $file = '/etc/samba/smb.conf';
    my $search = "   workgroup";
    my $workgroup = "   workgroup = $wg";

    # Open the file
    if (!open(FILE, "<$file")) {
        log_message("Could not open $file for writing");
        exit(1);
    }

    # Read and close the file
    my @lines = <FILE>;
    close(FILE);

    # Find the workgroup line
    my $index = 0;
    foreach (@lines) {
        last if ($_ =~ $search);
        $index++;
    }

    # Replace the line
    delete $lines[$index];
    $lines[$index] = $workgroup;
    $index++;
    splice(@lines,$index,0,"\n");

    if ($domaincontroller || $standalone) {
        # Find the modify_samba_config script
        log_message("Searching for modify_samba_config script locally");
        my $mod_script = `find $pwd -name modify_samba_config.pl`;
        if ($mod_script eq "") {
            log_message("Searching for modify_samba_config script globally--this could take a few minutes...");
            my @mod_scripts = `find / -name modify_samba_config.pl`;
            $mod_script = $mod_scripts[0];
            chomp($mod_script);
        } else {
            # Found locally--prepend current working directory to script name
            chomp($mod_script);
            my $len = length($mod_script);
            my $pwd = `pwd`;
            chomp($pwd);
            $mod_script = $pwd . "/" . substr($mod_script,2,$len);
        }
        if ($mod_script eq "") {
            log_message("modify_samba_config script not found--please complete the following items in smb.conf and retest:");
            log_message("   add share command");
            log_message("   delete share command");
            log_message("   addprinter command");
            log_message("   deleteprinter command");
        } else {
            system_logged("chmod +x $mod_script");
        }

        # Add the script lines
        my $share_def_line = "Share Definitions";
        my @systools;
        if ($backend eq "ldapsam") {
            # Add smbldap tool script lines
            $systools[0] = "   add user script = $idealx/smbldap-useradd -m \"%u\"\n";
            $systools[1] = "   delete user script = $idealx/smbldap-userdel -r \"%u\"\n";
            $systools[2] = "   add machine script = $idealx/smbldap-useradd \"%u\"\n";
            $systools[3] = "   add group script = $idealx/smbldap-groupadd -p \"%g\"\n";
            $systools[4] = "   add user to group script = $idealx/smbldap-groupmod -m \"%u\" \"%g\"\n";
            $systools[5] = "   delete user from group script = $idealx/smbldap-groupmod -x \"%u\" \"%g\"\n";
            $systools[6] = "   set primary group script = $idealx/smbldap-usermod -g \"%g\" \"%u\"\n";
            $systools[7] = "   add share command = $mod_script\n";
            $systools[8] = "   delete share command = $mod_script\n";
            $systools[9] = "   addprinter command = $mod_script\n";
            $systools[10] = "   deleteprinter command = $mod_script\n";
        } else {
            # Add system tool script lines
            $systools[0] = "   add user script = $sbin/useradd -m \"%u\"\n";
            $systools[1] = "   delete user script = $sbin/userdel -r \"%u\"\n";
            $systools[2] = "   add machine script = $sbin/useradd \"%u\"\n";
            $systools[3] = "   add group script = $sbin/groupadd -p \"%g\"\n";
            $systools[4] = "   add user to group script = $sbin/groupmod -m \"%u\" \"%g\"\n";
            $systools[5] = "   delete user from group script = $sbin/groupmod -x \"%u\" \"%g\"\n";
            $systools[6] = "   set primary group script = $sbin/usermod -g \"%g\" \"%u\"\n";
            $systools[7] = "   add share command = $mod_script\n";
            $systools[8] = "   delete share command = $mod_script\n";
            $systools[9] = "   addprinter command = $mod_script\n";
            $systools[10] = "   deleteprinter command = $mod_script\n";
        }

        # Check if entries already included
        my $toolsindex = 0;
        my $lastindex = $#lines;
        foreach (@lines) {
            last if ($_ =~ $systools[0]);
            $toolsindex++;
        }
        if ($toolsindex < $lastindex) {
            return;
        }

        # Find the Share Definitions line (bottom of global section)
        $index = 0;
        foreach (@lines) {
            last if ($_ =~ $share_def_line);
            $index++;
        }

        # Add the system tools lines
        $toolsindex = 10;
        foreach (@systools) {
            splice(@lines,$index,0,$systools[$toolsindex]);
            $toolsindex--;
        }
        $index = $index + 11;
        splice(@lines,$index,0,"\n");
    }

    # Add any optional global parameters
    $index++;
    foreach (@globalparams) {
        splice(@lines,$index,0,"   $_\n");
        $index++;
    }

    # Rewrite the file
    if (!open(FILE, ">$file")) {
        log_message("Could not open $file for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);
}

# Function to handle samba authorization
sub setup_samba_auth{
    my $role = shift;
    my $pw = shift;
    my $suffix = shift;
    my $rootdn = shift;
    
    # Edit the samba configuration file
    # If role = pdc, "domain logons" = yes
    #                "domain master" = yes
    #                "security" = user
    # otherwise,     "domain logons" = no
    #                "domain master" = false
    #                "security" = user

    # Set up some variables
    my $file = '/etc/samba/smb.conf';
    my $logons_line = "domain logons";
    my $master_line = "domain master";
    my $security_line = "security";
    my $share_def_line = "Share Definitions";
    my $pdc_logons_line = "\tdomain logons = yes";
    my $standalone_logons_line = "\tdomain logons = no";
    my $pdc_master_line = "\tdomain master = yes";
    my $standalone_master_line = "\tdomain master = false";
    my $pdc_security_line = "\tsecurity = user";
    my $standalone_security_line = "\tsecurity = user";

    # Open the conf file
    if (!open(FILE, "<$file")) {
        log_message("Could not open $file");
        exit(1);
    }

    # Read and close the file
    my @lines = <FILE>;
    close(FILE);

    # Find the domain logons line
    my $index = 0;
    foreach (@lines) {
        last if ($_ =~ $logons_line);
        $index++;
    }

    # Replace the domain logons line
    delete $lines[$index];
    if ($role eq "pdc") {
        $lines[$index] = $pdc_logons_line;
    } else {
        $lines[$index] = $standalone_logons_line;
    }
    $index++;
    splice(@lines,$index,0,"\n");

    # Find the domain master line
    $index = 0;
    foreach (@lines) {
        last if ($_ =~ $master_line);
        $index++;
    }

    # Replace the domain master line
    delete $lines[$index];
    if ($role eq "pdc") {
        $lines[$index] = $pdc_master_line;
    } else {
        $lines[$index] = $standalone_master_line;
    }
    $index++;
    splice(@lines,$index,0,"\n");

    # Find the security line
    $index = 0;
    foreach (@lines) {
        last if ($_ =~ $security_line);
        $index++;
    }

    # Replace the security line
    delete $lines[$index];
    if ($role eq "pdc") {
        $lines[$index] = $pdc_security_line;
    } else {
        # Check if user has requested share security
        # Only permitted for standalone
        foreach (@globalparams) {
            if ($_ =~ "security" || $_ =~ "Security") {
                if ($_ =~ "share" || $_ =~ "Share") {
                    $standalone_security_line = "\tsecurity = share";
                }
            }
        }
        $lines[$index] = $standalone_security_line;
    }
    $index++;
    splice(@lines,$index,0,"\n");

    # Enter the backend lines
    my @auth;
    if ($backend eq "ldapsam") {
        $auth[0] = "   ldap admin dn = $rootdn\n";
        $auth[1] = "   ldap suffix = $suffix\n";
        $auth[2] = "   ldap group suffix = ou=groups\n";
        $auth[3] = "   ldap user suffix = ou=users\n";
        $auth[4] = "   ldap machine suffix = ou=users\n";
        $auth[5] = "   ldap idmap suffix = ou=users\n";
        $auth[6] = "   passdb backend = ldapsam:ldap://localhost smbpasswd\n";
        $auth[7] = "   admin users = Administrator, root\n";
    } elsif ($backend eq "tdbsam") {
        $auth[0] = "   passdb backend = tdbsam\n";
        $auth[1] = "   admin users = Administrator, root\n";
    } elsif ($backend eq "smbpasswd") {
        $auth[0] = "   passdb backend = smbpasswd\n";
        $auth[1] = "   admin users = Administrator, root\n";
    }

    # Check if entries already included
    my $authindex = 0;
    my $lastindex = $#lines;
    foreach (@lines) {
        last if ($_ =~ $auth[0]);
        $authindex++;
    }
    if ($authindex >=$lastindex) {
        # Find the Share Definitions line (bottom of global section)
        $index = 0;
        foreach (@lines) {
            last if ($_ =~ $share_def_line);
            $index++;
        }

        # Add the auth lines
        $authindex = @auth - 1;
        foreach (@auth) {
            splice(@lines,$index,0,$auth[$authindex]);
            $authindex--;
        }
        $index = $index + @auth + 1;
        splice(@lines,$index,0,"\n");

        # Rewrite the conf file
        if (!open(FILE, ">$file")) {
            log_message("Could not open $file for writing");
            exit(1);
        }
        print FILE @lines;
        close(FILE);
    }

    # Stop the cache
    stop_nscd;

    # Set passwords
    system_logged("smbpasswd -w $pw");
    system_logged("smbpasswd -s -a root <<EOF
$pw
$pw
EOF");

    # Enable samba
    log_message("Enabling samba services");
    system_logged("chkconfig smb on");

    # Start samba
    log_message("Starting samba");
    start_samba("start");

    # Check if nmb needs to be started
    if (-e "/etc/init.d/nmb") {
        system_logged("/etc/init.d/nmb start");
    }
}

# Function to handle member setup
sub setup_member {
    my $pw = shift;

    # Get domain controller name for ldap.conf
    my $dc_ip = get_dc_ip($domain);
    my $dc_name = gethostbyaddr(inet_aton($dc_ip), AF_INET);

    # Handle smb.conf changes
    my $smb_file = '/etc/samba/smb.conf';

    # Set up search and replace variables
    my @search1 = ("domain logons =", "domain master =", "security = user");
    my @replace1 = ("domain logons = no\n", "domain master = false\n", "security = domain\n");

    # Open the smb conf file
    if (!open(FILE, "<$smb_file")) {
        log_message("Could not open $smb_file");
        exit(1);
    }

    # Read and close the smb file
    my @lines = <FILE>;
    close(FILE);

    # Find the lines and replace them
    for ($n=0; $n<3; $n++) {
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search1[$n]);
            $index++;
        }
        delete $lines[$index];
        $lines[$index] = $replace1[$n];
    }

    # Rewrite the smb conf file
    if (!open(FILE, ">$smb_file")) {
        log_message("Could not open $smb_file for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);

    # Join the domain
    my $joined=`net rpc join -w $domain -U$admin%$pw`;
    my $try=1;
    while ($joined != 0 && $try < 10) {
        log_message("Failed to join $domain, trying again");
        $try++;
        $joined=`net rpc join -w $domain -U$admin%$pw`;
    }
    if ($joined != 0) {
        log_message("Failed to join $domain after $try tries");
    } else {
        log_message("Successfully joined $domain");
    }
}

# =========================================
# Function to configure the openldap server
# =========================================

sub configure_ldap_server {
    my $host = shift;
    my $suffix = shift;
    my $rootdn = shift;
    my $rootpw = shift;
    chomp($host);

    # Handle the /etc/openldap/ldap.conf file
    # Set up some variables
    my $openldap_file = "/etc/openldap/ldap.conf";
    my @search = ("^BASE", "^HOST");
    my @replace = ("BASE $suffix\n", "HOST $host\n");

    # Open the /etc/openldap/ldap.conf file and read it in
    if (!open(FILE, "<$openldap_file")) {
        log_message("Could not open $openldap_file for input");
        exit(1);
    }
    my @lines = <FILE>;
    close(FILE);

    # Find the lines and replace them
    for ($n=0; $n<2; $n++) {
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search[$n]);
            $index++;
        }
        delete $lines[$index];
        $lines[$index] = $replace[$n];
    }

    # Rewrite the file
    if (!open(FILE, ">$openldap_file")) {
        log_message("Could not open $openldap_file for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);

    # Handle the /etc/openldap/slapd.conf file
    # Set up some variables
    my $slapd_file = "/etc/openldap/slapd.conf";
    my @search = ("^suffix", "^rootdn", "^rootpw");
    my $hashpw = `slappasswd -s $rootpw`;
    my @replace = ("suffix\t$suffix\n", "rootdn\t$rootdn\n", "rootpw\t$hashpw\n");

    # Open the /etc/openldap/slapd.conf file and read it in
    if (!open(FILE, "<$slapd_file")) {
        log_message("Could not open $slapd_file for input");
        exit(1);
    }
    my @lines = <FILE>;
    close(FILE);

    # Find the lines and replace them
    for ($n=0; $n<3; $n++) {
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search[$n]);
            $index++;
        }
        delete $lines[$index];
        $lines[$index] = $replace[$n];
    }

    # Rewrite the file
    if (!open(FILE, ">$slapd_file")) {
        log_message("Could not open $slapd_file for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);
}

# =========================================
# Function to configure the openldap client
# =========================================

sub configure_ldap_client {
    my $host = shift;
    my $suffix = shift;
    chomp($host);
    my $ldap_file = "/etc/ldap.conf";
    my @search = ("^base", "^host", "^ssl start_tls");
    my @replace = ("base $suffix\n", "host $host\n", "#ssl start_tls");

    # Open the /etc/ldap.conf file and read it in
    if (!open(FILE, "<$ldap_file")) {
        log_message("Could not open $ldap_file for input");
        exit(1);
    }
    my @lines = <FILE>;
    close(FILE);

    # Find the lines and replace them
    for ($n=0; $n<3; $n++) {
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search[$n]);
            $index++;
        }
        delete $lines[$index];
        $lines[$index] = $replace[$n];
    }

    # Rewrite the file
    if (!open(FILE, ">$ldap_file")) {
        log_message("Could not open $ldap_file for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);
}
