#!/usr/bin/perl -w

# Copyright(C) 2005, International Business Machines Corporation.  All
# Rights Reserved.

package test_tools;
use IO::Socket;
use IO::Socket::INET;

# Variables
use Exporter;
use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(
		open_logfile
		close_logfile
		system_logged
		log_message
		setup_groups
		get_rootdn
		get_host
		configure_nss
		configure_smbldap_tools
		create_file_shares
		create_printer
		create_print_shares
		delete_file_share
		delete_printer
		delete_print_share
		create_users
                create_guest
		delete_users
		delete_machine_users
                usage
		read_test_conf
		parse_multi_options
		get_dc_ip
		start_samba
		get_distro
		stop_nscd
		open_socket
		connect_socket
		test_printshares_client
		test_printshares_server
		test_printshares_standalone
		test_configuration
		test_user_creation
		test_share_creation
		test_privileges
		test_fileshares_1
		test_fileshares_2
		test_fileshares_3
		test_fileshares_4
		test_fileshares_5
		test_fileshares_6
		test_fileshares_7
		test_fileshares_8
		test_fileshares_9
		test_fileshares_10
		test_fileshares_11
		test_fileshares_12
		test_fileshares_13
		test_fileshares_14
		$domain
		$member
		$pdc
		@users
		%passwords
		%usertypes
		@fileshares
                %fsdirs
		%fsreaders
		%fswriters
		@printshares
		$backend
		@globalparams
		@fshareparams
		$logfile
		$idealx
		$sbin
		$testshare
		$printer
		$output
		$suffix
		$ldapsamtrusted
		$guest
		$test_passwd
		$sock_port
		$server_sock
		$client_sock
);

# Global variables
our ($domain, $member, $pdc);
our (@users, %passwords, %usertypes);
our (@fileshares, %fsdirs, %fsreaders, %fswriters, @printshares);
our ($logfile, $backend);
our (@globalparams, @fshareparams);
our $idealx = "/usr/local/sbin";
our $sbin = "/usr/sbin";
our $smbldap_tools = "smbldap-tools-0.9.1";
our $printer = "filepr";
our $output="/tmp/output.dat";
our $suffix;
our $ldapsamtrusted="no";
our $guest;
our $test_passwd;
our $sock_port = "7890";
our ($server_sock, $client_sock);

# Local variables
my $n = 0;

# Printshare test arrays
my @testpshares = ("pshareA", "pshareB");
my @testpfiles = ("/tmp/test1.dat", "/tmp/test2.dat", "/tmp/test3.dat");
my @testplines = (
    ["First line of test1\n", "Second line of test1\n", "Third line of test1\n"],
    ["First line of test2\n", "Second line of test2\n", "Third line of test2\n"],
    ["First line of test3\n", "Second line of test3\n", "Third line of test3\n"]
);



# Administrative user
my $admin = "root";

# ==================
# Logfile Management
# ==================

sub open_logfile {
    if (!open(LOGFILE, ">>$logfile")) {
        print "Could not open $logfile for appending\n";
    }
}

sub close_logfile {
    close(LOGFILE);
}

# ===================
# System Call Logging
# ===================

sub system_logged {
    my $sys_cmd = shift;
    print LOGFILE "==========\nRunning command: $sys_cmd\n";
    my $run_cmd = $sys_cmd." >> $logfile 2>&1";
    my $rc = system("$run_cmd");
    print LOGFILE "command returned $rc\n==========\n\n";
    return $rc;
}

# ===============
# Message logging
# ===============

sub log_message {
    my $message = shift;
    print LOGFILE "==========\n$message\n==========\n\n";
}

# =========================
# Function to set up groups 
# =========================

sub setup_groups {
    my $admin_cn = shift;
    my $nt_admin = shift;
    my $user_cn = shift;
    my $nt_user = shift;

    # Add unix groups
    if ($backend eq "ldapsam") {
        # These mapped groups are necessary for use in share definitions
        system_logged("$idealx/smbldap-groupadd -a $admin_cn");
        system_logged("$idealx/smbldap-groupadd -a $user_cn");
    } else {
        system_logged("$sbin/groupadd $admin_cn");
        system_logged("$sbin/groupadd $user_cn");
    }

    system_logged("net groupmap modify unixgroup=\"$admin_cn\" ntgroup=\"$nt_admin\"");
    system_logged("net groupmap modify unixgroup=\"$user_cn\" ntgroup=\"$nt_user\"");
}

# ===========================================
# Function to get the root distinguished name
# ===========================================

sub get_rootdn {
    my $suffix = shift;
    my $rootdn = "cn=Administrator," . $suffix;
    return $rootdn;
}

# ============================
# Function to get the hostname
# ============================

sub get_host {
    my $host = `hostname -f`;
    if ($host eq "") {
        log_message("Cannot determine hostname");
        log_message("Please ensure that /etc/hosts is set up correctly");
        exit(1);
    }
    return $host;
}

# =================================================
# Function to configure nss for ldap authentication
# =================================================

sub configure_nss {
    my $host = shift;

    # Configure nsswitch.conf
    my $nsswitch_conf = "/etc/nsswitch.conf";
    my @search = ("^passwd:", "^shadow:", "^group:");
    my @replace = ("passwd:      files passwd:file[SUCCESS=return] ldap\n", "shadow:      files passwd:file[SUCCESS=return] ldap\n", "group:       files passwd:file[SUCCESS=return] ldap\n");

    # Open the nsswitch.conf file and read it in
    if (!open(FILE, "<$nsswitch_conf")) {
        log_message("Could not open $nsswitch_conf for input");
        exit(1);
    }
    my @lines = <FILE>;
    close(FILE);

    # Find the lines and replace them
    for ($n=0; $n<3; $n++) {
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search[$n]);
            $index++;
        }
        delete $lines[$index];
        $lines[$index] = $replace[$n];
    }

    # Rewrite the file
    if (!open(FILE, ">$nsswitch_conf")) {
        log_message("Could not open $nsswitch_conf for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);

    # Successful return
    return(0);
}

# =======================================
# Function to configure the smbldap tools
# =======================================

sub configure_smbldap_tools {
    my $suffix = shift;
    my $rootdn = shift;
    my $rootpw = shift;
    my $domain = shift;

    # Set up some variables
    my @search;
    my @replace;
    my @lines;
    my $tools_dir = "/etc/opt/IDEALX/smbldap-tools";
    my $sid = `net getlocalsid`;
    my $sidlen = length($sid);
    my $siddex = index($sid,":");
    $siddex = $siddex+2;
    $sidlen = $sidlen-$siddex-1;
    $sid = substr($sid,$siddex,$sidlen);
    my $dnsdomain = `dnsdomainname`;
    chomp($dnsdomain);

    # Handle smbldap.conf file
    my $smbldap_file = "$tools_dir/smbldap.conf";
    @search = ("^SID=", "^sambaDomain=", "^ldapTLS=", "^suffix=", "^sambaUnixIdPooldn=", "^mailDomain=");
    @replace = ("SID=\"$sid\"\n", "sambaDomain=\"$domain\"\n", "ldapTLS=\"0\"\n", "suffix=\"$suffix\"\n", "sambaUnixIdPooldn=\"sambaDomainName=$domain,\${suffix}\"\n", "mailDomain=\"$dnsdomain\"\n");

    # Open the smbldap.conf file and read it in
    if (!open(FILE, "<$smbldap_file")) {
        log_message("Could not open $smbldap_file for input");
        exit(1);
    }
    @lines = <FILE>;
    close(FILE);

    # Find the lines and replace them
    for ($n=0; $n<6; $n++) {
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search[$n]);
            $index++;
        }
        delete $lines[$index];
        $lines[$index] = $replace[$n];
    }

    # Rewrite the file
    if (!open(FILE, ">$smbldap_file")) {
        log_message("Could not open $smbldap_file for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);

    # Handle smbldap_bind.conf file
    my $smbldap_bind_file = "$tools_dir/smbldap_bind.conf";
    @search = ("^slaveDN=", "^slavePw=", "^masterDN=", "^masterPw=");
    @replace = ("slaveDN=\"$rootdn\"\n", "slavePw=\"$rootpw\"\n", "masterDN=\"$rootdn\"\n", "masterPw=\"$rootpw\"\n");

    # Open the smbldap_bind.conf file and read it in
    if (!open(FILE, "<$smbldap_bind_file")) {
        log_message("Could not open $smbldap_bind_file for input");
        exit(1);
    }
    @lines = <FILE>;
    close(FILE);

    # Find the lines and replace them
    for ($n=0; $n<4; $n++) {
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search[$n]);
            $index++;
        }
        delete $lines[$index];
        $lines[$index] = $replace[$n];
    }

    # Rewrite the file
    if (!open(FILE, ">$smbldap_bind_file")) {
        log_message("Could not open $smbldap_bind_file for writing");
        exit(1);
    }
    print FILE @lines;
    close(FILE);
}

# =========================
# Functions to handle users
# =========================

sub create_users {
    my $group;
    my $passwd;

    foreach (@users) {
        if ($backend eq "ldapsam") {
            my $gid = get_ldap_gid($usertypes{$_});
            $passwd = $passwords{$_};
            system_logged("$idealx/smbldap-useradd -a -m -d /home/$_ -g $gid $_");
            system_logged("$idealx/smbldap-passwd $_ <<EOF
$passwd
$passwd
EOF");
        } else {
            if ($usertypes{$_} eq "admin") {
                $group = "root";
            } else {
                $group = "users";
            }        
            system_logged("useradd -m -d /home/$_ -g $group $_");
            $passwd = $passwords{$_};
            system_logged("smbpasswd -s -a $_ <<EOF
$passwd
$passwd
EOF");
        }
    }
}

sub create_guest {
    my $group;
    my $passwd;
    my @existingusers = `getent passwd`;
    my $currentuser;
    my $lastindex = $#existingusers;
    my $index=0;

    foreach (@existingusers) {
        $pos = index($_, ":");
        $currentuser = substr($_, 0, $pos);
        last if ($currentuser eq $guest);
        $index++;
    }

    # Create the guest if it does not already exist
    if ($index > $lastindex) {
        # Find the guest in the @users array
        $lastindex = $#users;
        $index = 0;
        foreach (@users) {
            last if ($_ eq $guest);
            $index++;
        }
        if ($index > $lastindex) {
            log_message("Could not find $guest in list of users");
        } else {
            # Create the user
            if ($backend eq "ldapsam") {
                my $gid = get_ldap_gid($usertypes{$_});
                $passwd = $passwords{$_};
                system_logged("$idealx/smbldap-useradd -a -m -d /home/$_ -g $gid $users[$index]");
            } else {
                if ($usertypes{$_} eq "admin") {
                    $group = "root";
                } else {
                    $group = "users";
                }        
                system_logged("useradd -m -d /home/$_ -g $group $users[$index]");
                $passwd = $passwords{$_};
            }
        }
    }
}

sub delete_users {
    foreach (@users) {
        if ($backend eq "ldapsam") {
            system_logged("$idealx/smbldap-userdel -r $_");
        } else {
            system_logged("userdel -r $_");
        }
    }
}

sub delete_machine_users {
    my @users = `getent passwd`;
    foreach (@users) {
        my $pos = index($_, ":");
        my $user = substr($_, 0, $pos);
        my $len = length($user);
        my $lastchar = substr($user, $len-1, $len-1);
        if ($lastchar eq "\$") {
            system_logged("userdel -r $user");
        }
    }
}

# =========================
# Functions for file shares
# =========================

sub create_file_shares {
    my $pw = shift;
    my $currentfs;
    my $fsdir;
    my $readers;
    my $writers;
    my $file = '/etc/samba/smb.conf';

    # Create the shares
    foreach (@fileshares) {
        $currentfs = $_;
        $fsdir = $fsdirs{$currentfs};
        $readers = $fsreaders{$currentfs};
        $writers = $fswriters{$currentfs};

        # Create the share directory and the share
        mkdir $fsdir;
        system_logged("net rpc share add $currentfs=$fsdir -C 'testshare' -U$admin%$pw");

        # Open the samba configuration file
        if (!open(FILE, "<$file")) {
            log_message("Could not open $file for writing");
            exit(1);
        }

        # Read and close the file
        my @lines = <FILE>;
        close(FILE);

        # Add read and write lists
        my $search = "testshare";
        my $index = 0;
        foreach (@lines) {
            last if ($_ =~ $search);
            $index++;
        }
        delete $lines[$index];
        if ($readers) {
            splice(@lines,$index,0,"\tread list                 = $readers\n");
            $index++;
        }
        if ($writers) {
            splice(@lines,$index,0,"\twrite list                = $writers\n");
            $index++;
        }

        # If there are additional parameters, add those
        my $pos;
        my $paramfs;
        my $param;
        my $entrylen;
        foreach (@fshareparams) {
            $entrylen = length($_);
            $pos = index($_, ":");
            $paramfs = substr($_, 0, $pos);
            if ($paramfs eq $currentfs) {
                $param = substr($_, $pos+1, $entrylen);
                splice(@lines,$index,0,"\t$param\n");
            }
        }

        # Rewrite the file
        if (!open(FILE, ">$file")) {
            log_message("Could not open $file for writing");
            exit(1);
        }

        print FILE @lines;
        close(FILE);
    }

    # Restart samba
    start_samba("restart");
}

sub delete_file_share {
    my $share = shift;
    my $pw = shift;
    system_logged("net rpc share delete $share -U$admin%$pw");
    start_samba("restart");
}

# ==========================
# Functions for print shares
# ==========================

# This subroutine creates a virtual printer ($printer) which is to be used
# in testing print shares, and which prints to a file ($output).

sub create_printer {
    my $port=11999;

    # Ensure that cups is running

    my $cups = `/etc/init.d/cups status`;
    if ($cups =~ "unused" || $cups =~ "stopped") {
        log_message("cupsd not running--starting");
        system_logged("/etc/init.d/cups start");
    }

    # Create a printer which prints to a file
    system_logged("lpadmin -p $printer -E -v socket://localhost:$port -m raw");
    my $pserve=IO::Socket::INET->new(LocalPort=>$port,Type=>SOCK_STREAM,Reuse=>1,Listen=>1) or die "Could not create print server--already running?\n";
    while ($pjob=$pserve->accept()) {
        open (JOB, ">>$output") or log_message("Could not open $output");
        while (<$pjob>) {
            print JOB "$_";
        }
        close(JOB);
        close($pjob);
    }
}

sub create_print_shares {
    my @conf_lines;
    my @share_lines;
    my $printspool = "/var/spool/samba";

    # Check if /var/spool/samba exists
    if (!-e "/var/spool/samba") {
        mkdir "/var/spool/samba";
    }

    # Create the shares
    foreach (@printshares) {
        # Set up share parameters
        @share_lines = (
            "\n",
            "## \[$_\]\n",
            "\[$_\]\n",
            "\tprinter\t\t= $printer\n",
            "\tdirectorymask\t= 0775\n",
            "\tcreatemask\t= 0664\n",
            "\tcomment\t\t= $printer\n",
            "\tprintable\t= Yes\n",
            "\tpath\t\t= $printspool\n",
            "\twritelist\t= \@ntadmin root\n",
        );

        # Read in smb.conf file
        if (!open(SMBCONF, "</etc/samba/smb.conf")) {
            log_message("Could not open smb.conf for reading");
        }
        @conf_lines = <SMBCONF>;
        close(SMBCONF);

        # Append print share
        foreach (@share_lines) {
            push(@conf_lines, $_);
        }

        # Rewrite smb.conf file
        if (!open(SMBCONF, ">/etc/samba/smb.conf")) {
            log_message("Could not open smb.conf for writing");
        }
        print SMBCONF @conf_lines;
        close(SMBCONF);
    }

    # Restart samba
    start_samba("restart");
}

sub delete_printer {
    system_logged("lpadmin -x $printer");
}

sub delete_print_share {
    my $share = shift;

    # Read in smb.conf file
    if (!open(SMBCONF, "</etc/samba/smb.conf")) {
        log_message("Could not open smb.conf for reading");
    }
    my @conf_lines = <SMBCONF>;
    close(SMBCONF);

    # Find the print share and delete the lines
    my $search = "## Section - \\\[$share\\\]\n",
    my $index = 0;
    foreach (@conf_lines) {
        last if ($_ =~ $search);
        $index++;
    }
    splice(@conf_lines, $index, 9);

    # Rewrite smb.conf file
    if (!open(SMBCONF, ">/etc/samba/smb.conf")) {
        log_message("Could not open smb.conf for writing")
    }
    print SMBCONF @conf_lines;
    close(SMBCONF);

    # Restart samba
    start_samba("restart");
}

# Function to handle the -? usage option for the config_*_test.pl scripts
sub usage {
    print "Usage: $0 -c {configuration file} -s {server ip address} [-l logfile]\n\n";

    print "    where:\n";
    print "        -c = test configuration file\n";
    print "        -s = test server ip address\n";
    print "        -l = filename for debug output (optional)\n";
    print "        -? = show this usage\n";
    return;
}

# This subroutine reads the specified test configuration file and places
# the contents into the following data structures:
#
#    DOMAINNAME is in $domain
#    PDC (yes or no) is in $pdc
#    USERS are in array @users
#    USER passwords are in hash $passwords
#        hash key is username from @users
#    USER attributes (user or admin) are in hash %usertypes
#        hash key is username from @users
#    FILESHARES are in array @fileshares
#    FILESHARE readers are in hash %fsreaders
#        hash key is fileshare name from @fileshares
#    FILESHARE writers are in has %fswriters
#        hash key is fileshare name from @fileshares
#    PRINTSHARES are in array @printshares
#    MEMBER is in $member
#    BACKEND is in $backend (includes $suffix if backend = ldapsam)
#    GLOBAL_PARAMS are in $globalparams
#    FSHARE_PARAMS are in @fshareparams
sub read_test_conf {
    my $testconf = shift;
    $suffix = "localdomain";

    my ($index, $pos, $entrylen, $itemlen, $size);
    my ($useritem, $currentuser, $currentpw);
    my ($fsitem, $currentfs, $currentdir);
    my ($rpos, $wpos, $rwitem, $rwlen, $rlist, $wlist);
    my ($rwtype, $currentps, $currentparam);

    # Open the file
    if (!open(TESTCONF, "<$testconf")) {
        log_message("Could not open $testconf for reading");
    }

    # Read the file
    my @lines = <TESTCONF>;
    $size = @lines;
    close(TESTCONF);
    if ($size == 0) {
        log_message("Empty configuration file");
        exit(1);
    }

    # -------------
    # Test Password
    # -------------

    # Find the password
    $index = 0;
    foreach (@lines) {
        last if ($_ =~ "^PASSWORD=" || $_ =~ "^password=");
        $index++;
    }
    if ($index == $size) {
        log_message("No PASSWORD entry exists");
        exit(1);
    }

    $entrylen = length($lines[$index]);
    $pos = index($lines[$index], "=");
    $test_passwd = substr($lines[$index],$pos+1,$entrylen);
    chomp($test_passwd);
    if (length($test_passwd) == 0) {
        log_message("Test password is blank--exiting");
        exit(1);
    }
    
    # ------------------
    # Domain Information
    # ------------------

    # Find the domain name
    $index = 0;
    foreach (@lines) {
        last if ($_ =~ "^DOMAINNAME=" || $_ =~ "^domainname=");
        $index++;
    }
    if ($index == $size) {
        log_message("No DOMAINNAME entry exists");
        exit(1);
    }
    $entrylen = length($lines[$index]);
    $pos = index($lines[$index], "=");
    $domain = substr($lines[$index],$pos+1,$entrylen);
    chomp($domain);
    if (length($domain) == 0) {
        log_message("Domain name is blank--exiting");
        exit(1);
    }
    
    # ---------------
    # PDC Information
    # ---------------

    $index = 0;
    foreach (@lines) {
        last if ($_ =~ "^PDC=" || $_ =~ "^pdc=");
        $index++;
    }
    if ($index == $size) {
        log_message("No PDC entry exists");
        exit(1);
    }
    $entrylen = length($lines[$index]);
    $pos = index($lines[$index], "=");
    $pdc = substr($lines[$index],$pos+1,$entrylen-1);
    chomp($pdc);
    if ($pdc eq "yes" || $pdc eq "Yes" || $pdc eq "YES") {
        $pdc = "yes";
    } elsif ($pdc eq "no" || $pdc eq "No" || $pdc eq "NO") {
        $pdc = "no";
    } else {
        log_message("Invalid PDC value \"$pdc\"");
        exit(1);
    }

    # ----------------
    # User Information
    # ----------------
    
    # Find the user lines
    foreach (@lines) {
        chomp($_);
        if ($_ =~ "^USER=" || $_ =~ "^user=") {
            $entrylen = length($_);
            $pos = index($_, "=");

            # Get the username, password, and usertype
            $useritem = substr($_,$pos+1,$entrylen);
            $itemlen = length($useritem);
            $pos = index($useritem, " ");
            if ($pos == -1) {
                log_message("***** Invalid format for $_, skipping entry *****");
            } else {
                # Remember the username
                $currentuser = substr($useritem, 0, $pos);

                # Get the password and usertype
                $useritem = substr($useritem, $pos+1, $itemlen);
                $itemlen = length($useritem);
                $pos = index($useritem, " ");
                if ($pos == -1) {
                    log_message("***** Invalid format for $_, skipping entry *****");
                } else {
                    # Store the username, password, and usertype
                    $currentpw = substr($useritem, 0, $pos);
                    $useritem = substr($useritem, $pos+1, $itemlen);
                    push(@users, $currentuser);
                    $passwords{$currentuser} = $currentpw;
                    $usertypes{$currentuser} = $useritem;
                }
            }
        }
    }
    
    # ---------------------
    # Fileshare Information
    # ---------------------
    
    # Find the fileshare lines
    foreach (@lines) {
        chomp($_);
        if ($_ =~ "^FILESHARE=" || $_ =~ "^fileshare=") {
            $entrylen = length($_);
            $pos = index($_, "=");
            
            # Get all share information
            $fsitem = substr($_,$pos+1,$entrylen);
            $itemlen = length($fsitem);

            # Separate out read/write lists
            # Multiple cases:
            #    There is only a read list
            #    There is only a write list
            #    There is both, read list first
            #    There is both, write list first
            # See if lists exist
            $rpos = index($fsitem, "r=");
            $wpos = index($fsitem, "w=");
            # Parse out the lists
            if ($rpos != -1 && $wpos != -1) {
                # Readlist and writelist exist
                if ($rpos < $wpos) {
                    $rwitem = substr($fsitem, $rpos, $entrylen);
                    $rwlen = length($rwitem);
                    $pos = index($rwitem, " ");
                    $rlist = substr($rwitem, 2, $pos-2);
                    $wlist = substr($rwitem, $pos+3, $rwlen);
                    $fsitem = substr($fsitem, 0, $rpos-1);
                } else {
                    $rwitem = substr($fsitem, $wpos, $entrylen);
                    $rwlen = length($rwitem);
                    $pos = index($rwitem, " ");
                    $wlist = substr($rwitem, 2, $pos-2);
                    $rlist = substr($rwitem, $pos+3, $rwlen);
                    $fsitem = substr($fsitem, 0, $wpos-1);
                }
            } elsif ($rpos !=  -1) {
                # Only readlist exists
                $rlist = substr($fsitem, $rpos+2, $entrylen);
                $wlist = "";
                $fsitem = substr($fsitem, 0, $rpos-1);
            } elsif ($wpos != -1) {
                # Only writelist exists
                $wlist = substr($fsitem, $wpos+2, $entrylen);
                $rlist = "";
                $fsitem = substr($fsitem, 0, $wpos-1);
            } else {
                # Neither exists
                $rlist = "";
                $wlist = "";
            }

            $entrylen = length($fsitem);
            $pos = index($fsitem, " ");
            if ($pos == -1) {
                    log_message("***** Invalid format for $_, skipping entry *****");
            } else {
                # Store the sharename and directory
                $currentfs = substr($fsitem, 0, $pos);
                $currentdir = substr($fsitem, $pos+1, $entrylen);
                push(@fileshares, $currentfs);
                $fsdirs{$currentfs} = $currentdir;

                # Store read/write lists
                $fsreaders{$currentfs} = $rlist;
                $fswriters{$currentfs} = $wlist;
            }
        }
    }
    
    # ----------------------
    # Printshare Information
    # ----------------------
    
    # Find the printshares
    foreach (@lines) {
        chomp($_);
        if ($_ =~ "^PRINTSHARE=" || $_ =~ "^printshare=") {
            $entrylen = length($_);
            $pos = index($_, "=");
            if ($pos == -1) {
                log_message("***** Invalid format for $_, skipping entry *****");
            } else {
                # Get the share name
                $currentps = substr($_, $pos+1, $entrylen);
                push(@printshares, $currentps);
            }
        }
    }
    
    # ------------------
    # Member Information
    # ------------------
    
    # Find the MEMBER value
    $index = 0;
    foreach (@lines) {
        last if ($_ =~ "^MEMBER=" || $_ =~ "^member=");
        $index++;
    }
    if ($index == $size) {
        log_message("No MEMBER entry exists");
        exit(1);
    }
    $entrylen = length($lines[$index]);
    $pos = index($lines[$index], "=");
    $member = substr($lines[$index],$pos+1,$entrylen-1);
    chomp($member);
    if ($member eq "rpc" || $member eq "Rpc" || $member eq "RPC") {
        $member = "rpc";
    } elsif ($member eq "ads" || $member eq "Ads" || $member eq "ADS") {
        $member = "ads";
    } elsif ($member eq "no" || $member eq "No" || $member eq "NO") {
        $member = "no";
    } else {
        log_message("Invalid MEMBER value $member");
        exit(1);
    }

    # -------------------
    # Backend Information
    # -------------------
    
    # Find the backend value
    $index = 0;
    foreach (@lines) {
        last if ($_ =~ "^BACKEND=" || $_ =~ "^backend=");
        $index++;
    }
    if ($index == $size) {
        log_message("No BACKEND entry exists");
        exit(1);
    }
    $entrylen = length($lines[$index]);
    $pos = index($lines[$index], "=");
    $backend = substr($lines[$index],$pos+1,$entrylen-1);
    chomp($backend);
    if ($backend eq "smbpasswd" || $backend eq "Smbpasswd" || $backend eq "SMBPASSWD") {
        $backend = "smbpasswd";
    } elsif ($backend eq "tdbsam" || $backend eq "Tdbsam" || $backend eq "TDBSAM") {
        $backend = "tdbsam";
    } elsif ($backend =~ "ldapsam" || $backend =~ "Ldapsam" || $backend =~ "LDAPSAM") {
        $index = index($backend, " ");
        $entrylen = length($backend);
        $suffix = substr($backend, $index+1, $entrylen);
        $backend = "ldapsam";
    } else {
        log_message("Invalid BACKEND value $backend");
        exit(1);
    }

    # ----------------------------
    # Global Parameter Information
    # ----------------------------
    
    # Find the global parameters
    foreach (@lines) {
        chomp($_);
        if ($_ =~ "^GLOBAL_PARAM=" || $_ =~ "^global_param=") {
            $entrylen = length($_);
            $pos = index($_, "=");
            $currentparam = substr($_, $pos+1, $entrylen);
            push(@globalparams, $currentparam);

            # Special handling for ldapsam:trusted=yes and guest account
            # Search for the parameter
            my $space = index($currentparam, " ");
            my $length = length($currentparam);
            my @substr;
            # Remove spaces from the parameter
            while ($space != -1) {
                $substr[0] = substr($currentparam, 0, $space);
                $substr[1] = substr($currentparam, $space+1, $length);
                $currentparam = $substr[0] . $substr[1];
                $length = length($currentparam);
                $space = index($currentparam, " ");
            }
            # If ldapsam:trusted=yes found, set ldapsamtrusted variable
            if ($currentparam =~ "ldapsam:trusted=yes") {
                $ldapsamtrusted = "yes";
            }
            # If guestaccount found, save the username
            if ($currentparam =~ "guestaccount") {
                $length = length($currentparam);
                $pos = index($currentparam, "=");
                $guest = substr($currentparam, $pos+1, $length);
                $length = length($guest);
                $pos = index($guest, "=");
                $guest = substr($guest, $pos+1, $length);
            }
        }
    }

    # -------------------------------
    # Fileshare Parameter Information
    # -------------------------------
    
    # Find the fileshare parameters
    foreach (@lines) {
        chomp($_);
        if ($_ =~ "^FSHARE_PARAM=" || $_ =~ "^fshare_param=") {
            $entrylen = length($_);
            $pos = index($_, "=");
            $currentparam = substr($_, $pos+1, $entrylen);
            push(@fshareparams, $currentparam);
        }
    }
}

sub parse_multi_options {
    my $pass = shift;
    my $len;
    my $pos=index($pass, "/");
    my @arr;
    my $n=0;
    my $substr;
    while ($pos > 0) {
        $len = length($pass);
        $arr[$n] = substr($pass,0,$pos);
        $pass = substr($pass,$pos+1,$len);
        $pos = index($pass, "/");
        $n++;
    }
    $arr[$n] = $pass;
    return @arr;
}

sub get_dc_ip {
    my $dom = shift;

    # Try net lookup dc first
    my $attempt = 0;
    my @ips;
    my $ip;
    while (!$ips[0] && $attempt < 6) {
        sleep 1;
        $attempt++;   
        @ips = `net lookup dc $dom`;
    }

    # If net lookup dc fails, try nmblookup
    if (!$ips[0]) {
        my $index;
        my $i = 0;
        @ips = `nmblookup $dom`;
        $ip = $ips[$i];
        while ($ip =~ "querying") {
            $i++;
            $ip = $ips[$i];
        }
        $index = index($ip, " ");
        $ip = substr($ip, 0, $index);
        if ($ip =~ "name_query failed") {
            log_message("Failed to find pdc ip address--test may fail");
        }
    } else {
        $ip = $ips[0];
        chomp ($ip);
    }
    return $ip;
}

sub get_ldap_gid {
    my $usertype = shift;

    # Get the groups
    my @groups = `getent group`;
    my $lastindex = $#groups;

    # Search for the proper group
    my $index = 0;
    my $search;
    if ($pdc eq "yes") {
        if ($usertype eq "admin") {
            $search = "domadmins";
        } else {
            $search = "domusers";
        }
    } else {
        if ($usertype eq "admin") {
            $search = "sambaadmins";
        } else {
            $search = "sambausers";
        }
    }
    foreach (@groups) {
        last if ($_ =~ $search);
        $index++;
    }
    if ($index > $lastindex) {
        log_message("Could not locate group with getent");
        return 0;
    }

    # Find the gid
    my $group = $groups[$index];
    my $len = length($group);
    $index = index($group, ":");
    my $substr = substr($group, $index+1, $len);
    $index = index($substr, ":");
    $substr = substr($substr, $index+1, $len);
    $index = index($substr, ":");
    my $gid = substr($substr, 0, $index);
    return $gid;
}

sub start_samba {
    my $mode = shift;
    my $attempt = 0;
    my $status;
    my $pos = 0;

    # Start or restart samba
    if ($mode eq "start") {
        system_logged("/etc/init.d/smb start");
    } else {
        system_logged("/etc/init.d/smb restart");
    }

    # Check status
    while ($attempt < 10) {
        $status = `/etc/init.d/smb status`;
        $pos = index($status, "running");
        if ($pos == -1) {
            log_message("Samba not running--checking again");
            sleep(2);
            $attempt++;
            if ($attempt == 10) {
                log_message("Samba did not start--tests will fail");
            }
        } else {
            log_message("Samba is running");
            sleep(5);
            return;
        }
    }
}

sub get_distro {
    my $release_data;
    my $pos;

    # Determine if SuSE or redhat
    if (-e "/etc/SuSE-release") {
        return "sles9";
    } elsif (-e "/etc/redhat-release") {
        # Determine redhat version
        $release_data=`cat /etc/redhat-release`;
        $pos = index($release_data, "release 4");
        if ($pos > -1) {
            return "rhel4";
        } else {
            $pos = index($release_data, "release 3");
            if ($pos > -1) {
                return "rhel3";
            } else {
                log_message("Test is running on unsupported distribution");
                return "unknown";
            }
        }
    } else {
        log_message("Test is running on unsupported distribution");
        return "unknown";
    }
}

sub stop_nscd {
    my $status = `/etc/init.d/nscd status`;
    my $pos = index($status, "running");
    if ($pos > -1) {
        log_message("Stopping nscd");
        system_logged("/etc/init.d/nscd stop");
    } else {
        log_message("nscd is not running");
    }
}

# ==========================
# Socket messaging functions
# ==========================

# open_socket is called from one of the two test scripts which require
# messaging.  The calling script uses the socket handle "server_sock"
# to send/receive messages.  The calling script should be started in
# background when run locally or in ABAT.
sub open_socket {
    # Create socket
    my $sock = new IO::Socket::INET(
        LocalPort => $sock_port,
        Proto => 'tcp',
        Listen => 1,
        Reuse => 1
    );
    if (!$sock) {
        log_message("Could not create socket on localhost:$sock_port");
    }

    # Accept connections
    $server_sock = $sock->accept();
}

# connect_socket is called from one of the two test scripts which require
# messaging.  This function requires the ip address of the socket server
# machine (the one which called create_socket).  The calling script uses
# the socket handle "client_sock" to send/receive messages.
sub connect_socket {
    my $remote_host = shift;

    # Connect to socket
    my $attempt = 1;
    while (!$client_sock && $attempt < 11) {
        $client_sock = new IO::Socket::INET(
            PeerAddr => $remote_host,
            PeerPort => $sock_port,
            Proto => 'tcp',
            Type => SOCK_STREAM
        );
        if (!client_sock) {
            log_message("Could not connect to $remote_host:$sock_port on attempt $attempt");
        }
        $attempt++;
        sleep(3);
    }
}

# ==================
# Printshare testing
# ==================

# This is the client side of printshare testing.  Note that this subroutine
# assumes sockets are already open.
sub test_printshares_client {
    my $server = shift;
    my $passwd = shift;
    my $buf;
    my $admin = "root";

    $ENV{"PASSWD"} = $passwd;
    for ($i=0; $i<3; $i++) {
        # Create a file to print
        if (!open(FILE, ">$testpfiles[$i]")) {
            log_message("Could not open $testpfiles[$i] for writing--test FAILED");
        } else {
            for ($j=0; $j<3; $j++) {
                # Print some lines to the file
                print FILE $testplines[$i][$j];
            }
            close(FILE);
            foreach (@testpshares) {
                system_logged("smbclient //$server/$_ -U$admin -c 'print $testpfiles[$i]'");

                # Tell server to run
                print $client_sock "RUN\n";

                # Wait for server to say it's OK to run again
                $buf = <$client_sock>;
                chomp($buf);
                while ($buf ne "FAILED" && $buf ne "PASSED") {
                    $buf = <$client_sock>;
                    chomp($buf);
                    sleep(2);
                }

                # Report results
                if ($buf eq "PASSED") {
                    log_message("Printing $testpfiles[$i] from $_ succeeded--test PASSED");
                } else {
                    log_message("Printing $testpfiles[$i] from $_ failed--test FAILED");
                }
            }
        }
    }

    # Clean up
    $ENV{"PASSWD"} = "";
    foreach (@testpfiles) {
        system_logged("rm $_");
    }
}

# This is the server side of printshare testing.  Note that this script
# assumes sockets are already open.
sub test_printshares_server {
    my $buf;
    my @lines;
    my $error;

    for (my $i=0; $i<3; $i++) {
        foreach (@testpshares) {
            # Wait for client to say it's OK to run
            $buf = <$server_sock>;
            chomp($buf);
            while ($buf ne "RUN") {
                $buf = <$server_sock>;
                chomp($buf);
                sleep(2);
            }

            # Give printer output time to show up
            my $attempt = 0;
            my $exists = "no";
            while ($attempt < 10 && $exists ne "yes") {
                if (-e $output) {
                    $exists = "yes";
                } else {
                    $attempt++;
                    log_message("Failed to find printer output on attempt $attempt");
                    sleep(2);
                }
            }

            # Get printer output
            if ($exists eq "yes") {
                @lines = `cat $output`;
                system_logged("rm $output");
                $error = 0;
                for (my $j=0; $j<3; $j++) {
                    if ($lines[$j] ne $testplines[$i][$j]) {
                        $error = 1;
                    }
                }
            } else {
                $error = 1;
            }

            # Report results
            if ($error != 0) {
                print $server_sock "FAILED\n";
            } else {
                print $server_sock "PASSED\n";
            }
        }
    }
}

# This is printshare testing for a standalone server
sub test_printshares_standalone {
    my $testconf = shift;

    # Save starting directory
    my $pwd = `pwd`;
    chomp($pwd);

    # Administrative user
    my $admin = "root";

    # Set up print shares
    my @localpshares = ("pshareA", "pshareB");

    # Read configuartion file
    read_test_conf($testconf);

    # Create some test files and test lines
    my @testfiles = ("$pwd/test1.dat", "$pwd/test2.dat", "$pwd/test3.dat");
    my @testlines = (
        ["First line of test1\n", "Second line of test1\n", "Third line of test1\n"],
        ["First line of test2\n", "Second line of test2\n", "Third line of test2\n"],
        ["First line of test3\n", "Second line of test3\n", "Third line of test3\n"]
    );

    # Give the printer time to warm up
    sleep(3);
    $ENV{"PASSWD"} = $test_passwd;
    for ($i=0; $i<3; $i++) {
        # Create a file to print
        if (!open(FILE, ">$testfiles[$i]")) {
            log_message("Could not open $testfiles[$i] for writing--test FAILED");
        } else {
            for ($j=0; $j<3; $j++) {
                # Print some lines to the file
                print FILE $testlines[$i][$j];
            }
        close(FILE);
        }
        foreach (@localpshares) {
            system_logged("smbclient //localhost/$_ -U$admin -c 'print $testfiles[$i]'");
            my @lines = `cat $output`;
            `rm $output`;
            $error = 0;
            for ($j=0; $j<3; $j++) {
                if ($lines[$j] ne $testlines[$i][$j]) {
                    $error = 1;
                }
            }
            if ($error != 0) {
                log_message("Error reading printout of $testfiles[$i]--test FAILED");
            } else {
                log_message("Success reading printout of $testfiles[$i]--test PASSED");
            }
        }
    }

    # Clean up
    $ENV{"PASSWD"} = "";
    foreach (@testfiles) {
        system_logged("rm $_");
    }
}
# =============================
# Configuration test subroutine
# =============================

sub test_configuration {
    my $testconf = shift;
    my $server = shift;

    # Read the configuration file
    read_test_conf($testconf);

    # Set up administrative user
    my $admin = "root";

    # Set up administrative password
    my $passwd = $test_passwd;

    # Subroutine variables
    my $lastindex;
    my $userindex;
    my $shareindex;
    my $currentuser;
    my $currentshare;
    my @existingusers;
    my @existingshares;

    # See if the configured users exist
    @existingusers = `net rpc user -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingusers;
    foreach (@users) {
        $userindex = 0;
        $currentuser = $_;
        log_message ("Testing for $currentuser");
        foreach (@existingusers) {
            chomp $_;
            last if ($_ eq $currentuser);
            $userindex++;
        }
        if ($userindex <= $lastindex) {
            log_message("$currentuser exists--test PASSED");
        } else {
            log_message("$currentuser not created--test FAILED");
        }
    }

    # See if the configured file shares exist
    @existingshares = `net rpc share -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingshares;
    foreach (@fileshares) {
        $shareindex = 0;
        $currentshare = $_;
        log_message ("Testing for $currentshare");
        foreach (@existingshares) {
            chomp $_;
            last if ($_ eq $currentshare);
            $shareindex++;
        }
        if ($shareindex <= $lastindex) {
            log_message("$currentshare exists--test PASSED");
        } else {
            log_message("$currentshare not created--test FAILED");
        }
    }

    # See if the configured print shares exist
    $lastindex = $#existingshares;
    foreach (@printshares) {
        $shareindex = 0;
        $currentshare = $_;
        log_message ("Testing for $currentshare");
        foreach (@existingshares) {
            chomp $_;
            last if ($_ eq $currentshare);
            $shareindex++;
        }
        if ($shareindex <= $lastindex) {
            log_message("$currentshare exists--test PASSED");
        } else {
            log_message("$currentshare not created--test FAILED");
        }
    }

    # Test domain name
    log_message("Testing for domain $domain");
    my $ipaddr = get_dc_ip($domain);
    chomp $ipaddr;
    log_message("IP address of domain $domain = $ipaddr");
    if (!$ipaddr) {
        log_message("Couldn't find domain--trying again");
        sleep 5;
        $ipaddr = get_dc_ip($domain);
    }
    if ($ipaddr) {
        log_message("Domain $domain successfully created--test PASSED");
    } else {
        log_message("Domain $domain does not exist--test FAILED");
    }
}

# ====================
# User test subroutine
# ====================

sub test_user_creation {
    my $testconf = shift;
    my $server = shift;

    # Read the configuration file
    read_test_conf($testconf);

    # Set up administrative user
    my $admin = "root";

    # Set up administrative password
    my $passwd = $test_passwd;

    # Subroutine variables
    my @testusers = ("testuser1", "testuser2", "testuser3", "testuser4", "testuser5", "testuser6", "testuser7", "testuser8", "testuser9", "testuser10");
    my $index;
    my $lastindex;
    my $currentuser;
    my @existingusers;
    my @createdusers;

    # Create the users
    # If the user already exists, skip the test
    @existingusers = `net rpc user -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingusers;
    foreach (@testusers) {
        $index = 0;
        $currentuser = $_;
        foreach (@existingusers) {
            chomp $_;
            last if ($_ eq $currentuser);
            $index++;
        }
        if ($index <= $lastindex) {
            log_message("User $currentuser exists--skipping\n");    
        } else {
            push(@createdusers, $_);
            system_logged("net rpc user add $_ -W $domain -I $server -U$admin%$passwd");
        }
    }

    # Test for users' existence
    @existingusers = `net rpc user -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingusers;
    foreach (@createdusers) {
        $index = 0;
        $currentuser = $_;
        foreach (@existingusers) {
            chomp $_;
            last if ($_ eq $currentuser);
            $index++;
        }
        if ($index <= $lastindex) {
            log_message("$currentuser created--test PASSED");
        } else {
            log_message("$currentuser not created--test FAILED");
        }
    }

    # Delete the users which were created
    foreach (@createdusers) {
        system_logged("net rpc user delete -W $domain -I $server $_ -U$admin%$passwd");
    }

    # Test for users' existence
    @existingusers = `net rpc user -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingusers;
    foreach (@createdusers) {
        $index = 0;
        $currentuser = $_;
        foreach (@existingusers) {
            chomp $_;
            last if ($_ eq $currentuser);
            $index++;
        }
        if ($index <= $lastindex) {
            log_message("$currentuser still exists--test FAILED");
        } else {
            log_message("$currentuser deleted--test PASSED");
        }
    }
}

# ======================
# Test shares subroutine
# ======================

sub test_share_creation {
    my $testconf = shift;
    my $server = shift;

    # Read the configuration file
    read_test_conf($testconf);

    # Set up administrative user
    my $admin = "root";

    # Set up administrative password
    my $passwd = $test_passwd;

    # Note that the share directories must be created by the server script
    my @testshares = ("testshare1", "testshare2", "testshare3", "testshare4", "testshare5", "testshare6", "testshare7", "testshare8", "testshare9", "testshare10");
    my %sharedirs;
    $sharedirs{"testshare1"} = "/tmp/sharedir1";
    $sharedirs{"testshare2"} = "/tmp/sharedir2";
    $sharedirs{"testshare3"} = "/tmp/sharedir3";
    $sharedirs{"testshare4"} = "/tmp/sharedir4";
    $sharedirs{"testshare5"} = "/tmp/sharedir5";
    $sharedirs{"testshare6"} = "/tmp/sharedir6";
    $sharedirs{"testshare7"} = "/tmp/sharedir7";
    $sharedirs{"testshare8"} = "/tmp/sharedir8";
    $sharedirs{"testshare9"} = "/tmp/sharedir9";
    $sharedirs{"testshare10"} = "/tmp/sharedir10";
    my $index;
    my $lastindex;
    my $currentshare;
    my @existingshares;
    my @createdshares;

    # Create the shares
    # If the share already exists, skip the test
    @existingshares = `net rpc share -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingshares;
    foreach (@testshares) {
        $index = 0;
        $currentshare = $_;
        foreach (@existingshares) {
            chomp $_;
            last if ($_ eq $currentshare);
            $index++;
        }
        if ($index <= $lastindex) {
            log_message("Share $currentshare exists--skipping\n");    
        } else {
            push(@createdshares, $_);
            system_logged("net rpc share add $_=$sharedirs{$_} -W $domain -I $server -U$admin%$passwd");
        }
    }

    # Test for shares' existence
    @existingshares = `net rpc share -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingshares;
    foreach (@createdshares) {
        $index = 0;
        $currentshare = $_;
        foreach (@existingshares) {
            chomp $_;
            last if ($_ eq $currentshare);
            $index++;
        }
        if ($index <= $lastindex) {
            log_message("$currentshare created--test PASSED");
        } else {
            log_message("$currentshare not created--test FAILED");
        }
    }

    # Delete the shares which were created
    foreach (@createdshares) {
        system_logged("net rpc share delete $_ -W $domain -I $server -U$admin%$passwd");
    }

    # Test for shares' existence
    @existingshares = `net rpc share -W $domain -I $server -U$admin%$passwd`;
    $lastindex = $#existingshares;
    foreach (@createdshares) {
        $index = 0;
        $currentshare = $_;
        foreach (@existingshares) {
            chomp $_;
            last if ($_ eq $currentshare);
            $index++;
        }
        if ($index <= $lastindex) {
            log_message("$currentshare still exists--test FAILED");
        } else {
            log_message("$currentshare deleted--test PASSED");
        }
    }
}

# ==========================
# Test privileges subroutine
# ==========================

sub test_privileges {
    my $testconf = shift;
    my $server = shift;

    # Read configuration file
    read_test_conf($testconf);

    # Set up administrative user and passwd
    my $admin_user = "root";
    my $admin_pw = $test_passwd;

    # Set up privileges user
    my $test_user = $users[0];
    my $test_pw = $passwords{$test_user};

    # Verify that the samba version has rights
    my $has_rights = has_rights();
    if ($has_rights == 1) {
        log_message("Starting privileges tests");
    } else {
        log_message("This version of smbd does not have net rpc rights");
        exit(0);
    }

    # Test Variables
    my $granted;
    my $users;
    my $index;
    my $exists;
    my @rights;
    my @privileges = ("SeMachineAccountPrivilege", "SeTakeOwnershipPrivilege", "SeBackupPrivilege", "SeRestorePrivilege", "SeRemoteShutdownPrivilege", "SePrintOperatorPrivilege", "SeAddUsersPrivilege", "SeDiskOperatorPrivilege");

    # Grant the rights to user1
    foreach (@privileges) {
        system_logged("net rpc rights grant $test_user $_ -U$admin_user%$admin_pw -S $server");
    }

    # Verify that the rights were granted
    @rights = `net rpc rights list $test_user -U$admin_user%$admin_pw -S $server`;
    foreach (@privileges) {
        $granted = "false";
        $current_priv = $_;
        foreach (@rights) {
            chomp($_);
            if ($_ eq $current_priv) {
                $granted = "true";
            }
        }
        if ($granted eq "true") {
            log_message("user1 was granted $_--test PASSED");
        } else {
            log_message("user1 not granted $_--test FAILED");
        }
    }

    # Check if user1 can actually add and delete a new user
    system_logged("net rpc user add newuser -W $domain -I $server -U$test_user%$test_pw");
    @users = `net rpc user -W $domain -I $server -U$$admin_user%$admin_pw`;
    $exists = "false";
    foreach (@users) {
        chomp($_);
        if ($_ eq "newuser") {
            $exists = "true";
        }
    }
    if ($exists eq "true") {
        log_message("$test_user successfully created the new user--test PASSED");
        system_logged("net rpc user delete newuser -W $domain -I $server -U$test_user%$test_pw");
    } else {
        log_message("$test_user failed to create the testuser--test FAILED");
    }

    # Revoke rights to user1
    foreach (@privileges) {
        system_logged("net rpc rights revoke $test_user $_ -U$admin_user%$admin_pw -S $server");
    }

    # Verify that the rights were revoked
    @rights = `net rpc rights list $test_user -U$admin_user%$admin_pw -S $server`;
    foreach (@privileges) {
        $granted = "false";
        $current_priv = $_;
        foreach (@rights) {
            chomp($_);
            if ($_ eq $current_priv) {
                $granted = "true";
            }
        }
        if ($granted eq "true") {
            log_message("$test_user still granted $_--test FAILED");
        } else {
            log_message("$test_user no longer granted $_--test PASSED");
        }
    }

    # Check if test user can still add and delete a new user
    system_logged("net rpc user add newuser -W $domain -I $server -U$test_user%$test_pw");
    @users = `net rpc user -W $domain -I $server -U$admin_user%$admin_pw`;
    $exists = "false";
    foreach (@users) {
        chomp($_);
        if ($_ eq "newuser") {
            $exists = "true";
        }
    }
    if ($exists eq "true") {
        log_message("$test_user successfully created the new user--test FAILED");
        `net rpc user delete newuser -W $domain -I $domain -U$test_user%$test_pw`;
    } else {
        log_message("$test_user failed to create the new user--test PASSED");
    }
}

sub has_rights {
    # Get the samba version (must be greater than 3.0.11)
    my $version = `smbd -V`;
    chomp $version;
    my $pos;
    my $pos2;
    my $v1;
    my $v2; 
    my $v3;

    # Parse out version numbers
    $pos = index($version, " ");
    $v1 = substr($version, $pos+1, 1);
    $pos = index($version, ".");
    $pos2 = index($version, ".", 10);
    $v2 = substr($version, $pos+1, 1);
    $v3 = substr($version, $pos2+1, 2);

    # Check for net rpc rights
    if ($v1 > 3) {
        return 1;
    } elsif ($v2 > 0) {
        return 1;
    } elsif ($v3 > 10) {
        return 1;
    }
    return 0;
}

# ==========================
# Fileshare test subroutines
# ==========================

sub test_fileshares_1 {
    my $server = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";
    
    # ============
    # ShareA tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareA failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareA failed--test FAILED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareA -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareA succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareA failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareA -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareA succeeded--test FAILED");
    } else {
        log_message("user2 smbclient write to shareA failed--test PASSED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareB succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareB failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write to shareB failed--test PASSED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareB -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareB succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareB failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareB -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareB succeeded--test FAILED");
    } else {
        log_message("user2 smbclient write to shareB failed--test PASSED");
    }
    
    # ============
    # ShareC tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareC failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareC failed--test FAILED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareC -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareC succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareC failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareC -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareC succeeded--test PASSED");
    } else {
        log_message("user2 smbclient write to shareC failed--test FAILED");
    }
    
    # ============
    # ShareD tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareD failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write to shareD failed--test PASSED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareD -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareD failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareD -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareD succeeded--test PASSED");
    } else {
        log_message("user2 smbclient write to shareD failed--test FAILED");
    }
}

sub test_fileshares_2 {
    my $server = shift;
    my $rc = 0;
    my $testmount = "/tmp/testmount";
    my $testfile = "$testmount/test.dat";
    mkdir $testmount;
    
    # ============
    # ShareA tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareA $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareA readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareA failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareA $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareA read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareA failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareA $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareA readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareA failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareA $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareA read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to shareA failed--test FAILED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareB $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareB readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareB failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareB $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareB read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareB failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareB $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareB readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareB failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareB $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareB read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to shareB failed--test FAILED");
    }
    # ============
    # ShareC tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareC $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareC readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareC failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareC $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareC read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareC failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareC $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareC readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareC failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareC $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareC read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to shareC failed--test FAILED");
    }
    # ============
    # ShareD tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareD $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareD readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareD failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareD $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareD read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareD failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareD $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareD readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareD failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareD $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareD read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to failed--test FAILED");
    }
}

sub test_fileshares_3 {
    my $server = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";
    
    # ============
    # ShareA tests
    # ============
    
    # connect to available=yes share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'pwd'");
    if ($rc == 0) {
        log_message("user1 smbclient connection to shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient connection to shareA failed--test FAILED");
    }
    
    # read from available=yes share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareA failed--test FAILED");
    }
    
    # write to available=yes share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareA failed--test FAILED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # connect to available=no share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'pwd'");
    if ($rc == 0) {
        log_message("user1 smbclient connection to shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient connection to shareB failed--test PASSED");
    }
    
    # read from available=no share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read from shareB failed--test PASSED");
    }
    
    # write to available=no share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write to shareB failed--test PASSED");
    }
}

sub test_fileshares_4 {
    my $server = shift;
    my $rc = 0;
    my $testmount = "/tmp/testmount";
    my $testfile = "$testmount/test.dat";
    mkdir $testmount;
    
    # ============
    # ShareA tests
    # ============
    
    # connect to available=yes share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -t cifs //$server/shareA $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 mount connection to shareA succeeded--test PASSED");
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount connection to shareA failed--test FAILED");
    }
    
    # read and write available=yes share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -t cifs //$server/shareA $testmount -o username=user1");
    if ($rc == 0) {
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount of shareA failed--test FAILED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # connect to available=no share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -t cifs //$server/shareB $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 mount connection to shareB succeeded--test FAILED");
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount connection to shareB failed--test PASSED");
    }
    
    # read and write available=no share
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -t cifs //$server/shareB $testmount -o username=user1");
    if ($rc == 0) {
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount of shareB failed--test PASSED");
    }
}

sub test_fileshares_5 {
    my $server = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";
    
    # ============
    # ShareA tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareA failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareA succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write to shareA failed--test PASSED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient connect to shareA without password succeeded--test FAILED");
    } else {
        log_message("user1 smbclient connect to shareA without password failed--test PASSED");
    }
    
    # read as testguest using a password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read from shareA succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read from shareA failed--test FAILED");
    }
    
    # write as testguest using a password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write to shareA succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write to shareA failed--test PASSED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient connect to shareA without password succeeded--test FAILED");
    } else {
        log_message("testguest smbclient connect to shareA without password failed--test PASSED");
    }
    
    # Connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient connect to shareA without password succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient connect to shareA without password failed--test PASSED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareB succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareB failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareB succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareB failed--test FAILED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient connect to shareB without password succeeded--test FAILED");
    } else {
        log_message("user1 smbclient connect to shareB without password failed--test PASSED");
    }
    
    # read as testguest using a password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read from shareB succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read from shareB failed--test FAILED");
    }
    
    # write as testguest using a password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write to shareB succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write to shareB failed--test FAILED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient connect to shareB without password succeeded--test FAILED");
    } else {
        log_message("testguest smbclient connect to shareB without password failed--test PASSED");
    }
    
    # Connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient connect to shareB succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient connect to shareB failed--test PASSED");
    }
    
    # ============
    # ShareC tests
    # ============
    
    # connect as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient connect to shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient connect to shareC failed--test FAILED");
    }

    # connect as testguest
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient connect to shareC succeeded--test PASSED");
    } else {
        log_message("testguest smbclient connect to shareC failed--test FAILED");
    }

    # connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient connect to shareC succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient connect to shareC failed--test PASSED");
    }

    # ============
    # shareD tests
    # ============

    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareD failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write to shareD failed--test PASSED");
    }

    # read as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read without password from shareD failed--test FAILED");
    }
    
    # write as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write to shareD failed--test PASSED");
    }

    # read as testguest
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read from shareD failed--test FAILED");
    }
    
    # write as testguest
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write to shareD succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write to shareD failed--test PASSED");
    }

    # read as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read without password from shareD failed--test FAILED");
    }
    
    # write as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write without password to shareD succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write without password to shareD failed--test PASSED");
    }

    # read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareD failed--test FAILED");
    }
    
    # write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U nonuser -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient write to shareD succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient write to shareD failed--test PASSED");
    }

    # ============
    # shareE tests
    # ============

    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareE -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareE succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareE failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareE -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareE succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareE failed--test FAILED");
    }

    # read as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareE -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareE succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read without password from shareE failed--test FAILED");
    }
    
    # write as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareE -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write without password to shareE succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write without password to shareE failed--test FAILED");
    }

    # read as testguest
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareE -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read from shareE succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read from shareE failed--test FAILED");
    }
    
    # write as testguest
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareE -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write to shareE succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write to shareE failed--test FAILED");
    }

    # read as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareE -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareE succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read without password from shareE failed--test FAILED");
    }
    
    # write as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareE -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write without password to shareE succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write without password to shareE failed--test FAILED");
    }

    # read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareE -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareE succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareE failed--test FAILED");
    }
    
    # write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareE -U nonuser -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient write to shareE succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient write to shareE failed--test FAILED");
    }

    # ============
    # shareF tests
    # ============

    # read as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareF -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareF succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read without password from shareF failed--test FAILED");
    }
    
    # write as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareF -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write without password to shareF succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write without password to shareF failed--test PASSED");
    }

    # read as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareF -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareF succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read without password from shareF failed--test FAILED");
    }
    
    # write as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareF -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write without password to shareF succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write without password to shareF failed--test PASSED");
    }

    # read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareF -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareF succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareF failed--test FAILED");
    }
    
    # write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareF -U nonuser -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient write to shareF succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient write to shareF failed--test PASSED");
    }

    # ============
    # shareG tests
    # ============

    # read as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareG -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareG succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read without password from shareG failed--test FAILED");
    }
    
    # write as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareG -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write without password to shareG succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write without password to shareG failed--test FAILED");
    }

    # read as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareG -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareG succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read without password from shareG failed--test FAILED");
    }
    
    # write as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareG -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write without password to shareG succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write without password to shareG failed--test FAILED");
    }

    # read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareG -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareG succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareG failed--test FAILED");
    }
    
    # write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareG -U nonuser -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient write to shareG succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient write to shareG failed--test FAILED");
    }
}

sub test_fileshares_6 {
    my $server = shift;
    my $rc = 0;
    my $testmount = "/tmp/testmount";
    my $testfile = "$testmount/test.dat";
    mkdir $testmount;
    
    # ============
    # ShareA tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbmount //$server/shareA $testmount -o username=user1 ro");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareA readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount readonly to shareA failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("smbmount //$server/shareA $testmount -o username=user1 rw");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareA read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test FAILED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount read/write to shareA failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbmount //$server/shareA $testmount -o username=user2 ro");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareA readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount readonly to shareA failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("smbmount //$server/shareA $testmount -o username=user2 rw");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareA read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount read/write to shareA failed--test FAILED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbmount //$server/shareB $testmount -o username=user1 ro");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareB readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount readonly to shareB failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("smbmount //$server/shareB $testmount -o username=user1 rw");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareB read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount read/write to shareB failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbmount //$server/shareB $testmount -o username=user2 ro");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareB readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount readonly to shareB failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("smbmount //$server/shareB $testmount -o username=user2 rw");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareB read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount read/write to shareB failed--test FAILED");
    }

    # ============
    # ShareC tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbmount //$server/shareC $testmount -o username=user1 ro");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareC readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount readonly to shareC failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("smbmount //$server/shareC $testmount -o username=user1 rw");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareC read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test FAILED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount read/write to shareC failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbmount //$server/shareC $testmount -o username=user2 ro");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareC readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount readonly to shareC failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("smbmount //$server/shareC $testmount -o username=user2 rw");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareC read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test FAILED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount read/write to shareC failed--test FAILED");
    }
    # ============
    # ShareD tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbmount //$server/shareD $testmount -o username=user1 ro");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareD readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount readonly to shareD failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("smbmount //$server/shareD $testmount -o username=user1 rw");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareD read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user1 mount read/write to shareD failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbmount //$server/shareD $testmount -o username=user2 ro");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareD readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount readonly to shareD failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("smbmount //$server/shareD $testmount -o username=user2 rw");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareD read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test FAILED");
        }
        system_logged("smbumount $testmount");
    } else {
        log_message("user2 mount read/write to failed--test FAILED");
    }
}

sub test_fileshares_7 {
    my $server = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";
    
    # ============
    # ShareA tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareA failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareA failed--test FAILED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareA -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareA succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareA failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareA -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareA succeeded--test PASSED");
    } else {
        log_message("user2 smbclient write to shareA failed--test FAILED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareB succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareB failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write to shareB failed--test PASSED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareB -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareB succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareB failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareB -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareB succeeded--test FAILED");
    } else {
        log_message("user2 smbclient write to shareB failed--test PASSED");
    }
    
    # ============
    # ShareC tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareC failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareC failed--test FAILED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareC -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareC succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareC failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareC -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareC succeeded--test PASSED");
    } else {
        log_message("user2 smbclient write to shareC failed--test FAILED");
    }
    
    # ============
    # ShareD tests
    # ============
    
    # read as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from shareD failed--test FAILED");
    }
    
    # write as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to shareD failed--test FAILED");
    }
    
    # read as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareD -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from shareD failed--test FAILED");
    }
    
    # write as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/shareD -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to shareD succeeded--test PASSED");
    } else {
        log_message("user2 smbclient write to shareD failed--test FAILED");
    }
}

sub test_fileshares_8 {
    my $server = shift;
    my $rc = 0;
    my $testmount = "/tmp/testmount";
    my $testfile = "$testmount/test.dat";
    mkdir $testmount;
    
    # ============
    # ShareA tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareA $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareA readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareA failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareA $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareA read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareA failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareA $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareA readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareA failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareA $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareA read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to shareA failed--test FAILED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareB $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareB readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareB failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareB $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareB read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareB failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareB $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareB readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareB failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareB $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareB read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to shareB failed--test FAILED");
    }
    # ============
    # ShareC tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareC $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareC readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareC failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareC $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareC read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareC failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareC $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareC readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareC failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareC $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareC read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to shareC failed--test FAILED");
    }
    # ============
    # ShareD tests
    # ============
    
    # mount readonly as user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("mount -r -t cifs //$server/shareD $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareD readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount readonly to shareD failed--test FAILED");
    }
    
    # mount read/write as user1
    $rc = system_logged("mount -w -t cifs //$server/shareD $testmount -o username=user1");
    if ($rc == 0) {
        log_message("user1 successfully mounted shareD read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user1 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user1 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user1 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user1 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user1 mount read/write to shareD failed--test FAILED");
    }
    
    # Mount readonly as user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("mount -r -t cifs //$server/shareD $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareD readonly--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test PASSED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test FAILED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test PASSED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount readonly to shareD failed--test FAILED");
    }
    
    # mount read/write as user2
    $rc = system_logged("mount -w -t cifs //$server/shareD $testmount -o username=user2");
    if ($rc == 0) {
        log_message("user2 successfully mounted shareD read/write--continuing");
        if (open(FILE, "<$testfile")) {
            log_message("user2 opened $testfile for reading--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for reading--test FAILED");
        }
        if (open(FILE, ">$testfile")) {
            log_message("user2 opened $testfile for writing--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for writing--test FAILED");
        }
        if (open(FILE, ">>$testfile")) {
            log_message("user2 opened $testfile for appending--test PASSED");
            close(FILE);
        } else {
            log_message("user2 could not open $testfile for appending--test FAILED");
        }
        system_logged("umount $testmount");
    } else {
        log_message("user2 mount read/write to failed--test FAILED");
    }
}

# Test connecting to home directories
sub test_fileshares_9 {
    my $server = shift;
    my $conf = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";

    # Get users and passwords
    read_test_conf($conf);

    # connect to each user
    foreach (@users) {
        # set password
        $ENV{"PASSWD"} = $passwords{$_};

        # write test
        $rc = system_logged("smbclient //$server/$_ -U $_ -c 'put $tmpfile $testfile'");
        if ($rc == 0) {
            log_message("$_ smbclient write to /home/$_ succeeded--test PASSED");
        } else {
            log_message("$_ smbclient write to /home/$_ failed--test FAILED");
        }

        # read test
        $rc = system_logged("smbclient //$server/$_ -U $_ -c 'get $testfile $tmpfile'");
        if ($rc == 0) {
            log_message("$_ smbclient read from /home/$_ succeeded--test PASSED");
        } else {
            log_message("$_ smbclient read from /home/$_ failed--test FAILED");
        }
    }

    # connect to homes
    foreach (@users) {
        # set password
        $ENV{"PASSWD"} = $passwords{$_};

        # write test
        $rc = system_logged("smbclient //$server/homes -U $_ -c 'put $tmpfile $testfile'");
        if ($rc == 0) {
            log_message("$_ smbclient write to homes succeeded--test PASSED");
        } else {
            log_message("$_ smbclient write to homes failed--test FAILED");
        }

        # read test
        $rc = system_logged("smbclient //$server/homes -U $_ -c 'get $testfile $tmpfile'");
        if ($rc == 0) {
            log_message("$_ smbclient read from homes succeeded--test PASSED");
        } else {
            log_message("$_ smbclient read from homes failed--test FAILED");
        }
    }
}

# Test for cat.  The server setup script should create a test.dat file in 
# each share with the line "TESTING" in it.  See share_tests_server.pl for
# an example.
sub test_fileshares_10 {
    my $server = shift;
    my $rc = 0;
    my $testmount = "/tmp/testmount";
    my $testfile = "$testmount/test.dat";
    mkdir $testmount;
    my $cat_output;
    my $testline = "TESTING";
    my @share = ("shareA", "shareB", "shareC", "shareD");

    # Start of tests
    foreach (@share) {
        # mount as user1
        $ENV{"PASSWD"} = "passwd1";
        $rc = system_logged("mount -w -t cifs //$server/$_ $testmount -o username=user1");
        if ($rc == 0) {
            log_message("user1 successfully mounted $_--continuing");
            $cat_output = `cat $testfile`;
            chomp($cat_output);
            if ($cat_output eq $testline) {
                log_message("user1 cat'ed $testfile successfully--test PASSED");
           } else {
                log_message("user1 did not cat $testfile--test FAILED");
           }
           system_logged("umount $testmount")
        } else {
            log_message("user1 could not mount $_--test FAILED");
        }

        # mount as user2
        $ENV{"PASSWD"} = "passwd2";
        $rc = system_logged("mount -w -t cifs //$server/$_ $testmount -o username=user2");
        if ($rc == 0) {
            log_message("user2 successfully mounted $_--continuing");
            $cat_output = `cat $testfile`;
            chomp($cat_output);
            if ($cat_output eq $testline) {
                log_message("user2 cat'ed $testfile successfully--test PASSED");
            } else {
                log_message("user2 did not cat $testfile--test FAILED");
            }
            system_logged("umount $testmount")
        } else {
            log_message("user2 could not mount $_--test FAILED");
        }
    }
}

# Test map to guest = never.  The server setup script should create a
# test.dat file in each share.  See share_tests_server.pl for an example.
sub test_fileshares_11 {
    my $server = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";
    
    # ============
    # ShareA tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareA failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write with password to shareA failed--test FAILED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareA succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareA failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareA succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareA failed--test FAILED");
    }
    
    # write as testguest with  password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareA succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareA failed--test PASSED");
    }

    # Read as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareA succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareA failed--test PASSED");
    }
    
    # Read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read without password from shareA succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read without password from shareA failed--test PASSED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareB succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareB failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write with password to shareB failed--test PASSED");
    }
    
    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareB failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareB succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareB failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareB succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareB failed--test PASSED");
    }
    
    # Connect as testguest without password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareB succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareB failed--test PASSED");
    }
    
    # Read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareB succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read from shareB failed--test PASSED");
    }
    
    # ============
    # ShareC tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareC failed--test FAILED");
    }

    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write with password to shareC failed--test FAILED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareC succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareC failed--test PASSED");
    }

    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareC succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareC failed--test FAILED");
    }

    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareC succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareC failed--test PASSED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareC succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareC failed--test PASSED");
    }

    # Connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareC succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read from shareC failed--test PASSED");
    }

    # ============
    # shareD tests
    # ============

    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareD failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write with password to shareD failed--test PASSED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareD failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareD failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write with password to shareD failed--test FAILED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareD succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareD failed--test PASSED");
    }
    
    # Connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareD succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read from shareD failed--test PASSED");
    }
}

# Test map to guest = bad user.  The server setup script should create a
# test.dat file in each share.  See share_tests_server.pl for an example.
sub test_fileshares_12 {
    my $server = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";
    
    # ============
    # ShareA tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareA failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write with password to shareA failed--test FAILED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareA succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareA failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareA succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareA failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareA succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareA failed--test PASSED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareA succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareA failed--test PASSED");
    }
    
    # Connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareA succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read from shareA failed--test PASSED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareB succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareB failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write with password to shareB failed--test PASSED");
    }
    
    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareB failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareB succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareB failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareB succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareB failed--test PASSED");
    }
    
    # Connect as testguest without password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareB succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareB failed--test PASSED");
    }
    
    # Connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareB succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read from shareB failed--test PASSED");
    }
    
    # ============
    # ShareC tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareC failed--test FAILED");
    }

    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write with password to shareC failed--test FAILED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareC succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareC failed--test PASSED");
    }

    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareC succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareC failed--test FAILED");
    }

    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareC succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareC failed--test PASSED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareC succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareC failed--test PASSED");
    }

    # Connect as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareC succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareC failed--test FAILED");
    }

    # Write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U nonuser -c 'put $tmpfile$testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient write to shareC succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient write to shareC failed--test PASSED");
    }

    # ============
    # shareD tests
    # ============

    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareD failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write with password to shareD failed--test PASSED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareD failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareD failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write with password to shareD failed--test FAILED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareD succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareD failed--test PASSED");
    }
    
    # read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareD failed--test FAILED");
    }
    
    # write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U nonuser -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient write to shareD succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient write to shareD failed--test FAILED");
    }
}

# Test map to guest = bad password.  The server setup script should create a
# test.dat file in each share.  See share_tests_server.pl for an example.
sub test_fileshares_13 {
    my $server = shift;
    my $rc = 0;
    my $testfile = "test.dat";
    my $tmpfile = "/tmp/$testfile";
    
    # ============
    # ShareA tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareA failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareA succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write with password to shareA failed--test FAILED");
    }

    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareA succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareA failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareA succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareA failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareA succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareA failed--test PASSED");
    }

    # Connect as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareA succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareA failed--test PASSED");
    }
    
    # Read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareA -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareA succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read from shareA failed--test PASSED");
    }
    
    # ============
    # ShareB tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareB succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareB failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write with password to shareB failed--test PASSED");
    }
    
    # Connect as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareB succeeded--test FAILED");
    } else {
        log_message("user1 smbclient read without password from shareB failed--test PASSED");
    }
    
    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareB succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareB failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareB succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareB failed--test PASSED");
    }
    
    # Connect as testguest without password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareB -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareB succeeded--test FAILED");
    } else {
        log_message("testguest smbclient read without password from shareB failed--test PASSED");
    }
    
    # Read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareB -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareB succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read without password from shareB failed--test PASSED");
    }
    
    # ============
    # ShareC tests
    # ============
    
    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareC failed--test FAILED");
    }

    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write with password to shareC failed--test FAILED");
    }

    # read as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareC succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read without password from shareC failed--test FAILED");
    }

    # write as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write without password from shareC succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write without password from shareC failed--test PASSED");
    }

    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareC succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareC failed--test FAILED");
    }

    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareC succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write with password to shareC failed--test PASSED");
    }

    # read as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareC succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read without password from shareC failed--test FAILED");
    }

    # write as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write without password to shareC succeeded--test FAILED");
    } else {
        log_message("testguest smbclient write without password to shareC failed--test PASSED");
    }

    # Read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareC succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareC failed--test FAILED");
    }

    # Write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareC -U nonuser -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareC succeeded--test FAILED");
    } else {
        log_message("non-existing user smbclient read from shareC failed--test PASSED");
    }

    # ============
    # shareD tests
    # ============

    # read as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read with password from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read with password from shareD failed--test FAILED");
    }
    
    # write as user1 with password
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write with password to shareD succeeded--test FAILED");
    } else {
        log_message("user1 smbclient write with password to shareD failed--test PASSED");
    }

    # read as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read without password from shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read without password from shareD failed--test FAILED");
    }
    
    # write as user1 without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write without password to shareD succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write without password to shareD failed--test FAILED");
    }

    # read as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read with password from shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read with password from shareD failed--test FAILED");
    }
    
    # write as testguest with password
    $ENV{"PASSWD"} = "passwdtg";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write with password to shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write with password to shareD failed--test FAILED");
    }

    # read as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("testguest smbclient read without password from shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient read without password from shareD failed--test FAILED");
    }
    
    # write as testguest without password
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U testguest -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("testguest smbclient write without password to shareD succeeded--test PASSED");
    } else {
        log_message("testguest smbclient write without password to shareD failed--test FAILED");
    }

    # read as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U nonuser -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient read from shareD succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient read from shareD failed--test FAILED");
    }

    # write as non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/shareD -U nonuser -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("non-existing user smbclient write to shareD succeeded--test PASSED");
    } else {
        log_message("non-existing user smbclient write to shareD failed--test FAILED");
    }
}

sub test_fileshares_14 {
    my $server = shift;
    my $rc = 0;
    my $testfile;
    my $tmpfile;
    
    # Connect to non-existing user
    $ENV{"PASSWD"} = "";
    $rc = system_logged("smbclient //$server/nonuser -U nonuser -c 'pwd'");
    if ($rc == 0) {
        log_message("user1 smbclient connection to non-existing user succeeded--test FAILED");
    } else {
        log_message("user1 smbclient connection to non-existing user failed--test PASSED");
    }
    
    # read from user1
    $testfile = "test1.dat";
    $tmpfile = "/tmp/$testfile";
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/user1 -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user1 smbclient read from user1 succeeded--test PASSED");
    } else {
        log_message("user1 smbclient read from user1 failed--test FAILED");
    }
    
    # write to user1
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/user1 -U user1 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user1 smbclient write to user1 succeeded--test PASSED");
    } else {
        log_message("user1 smbclient write to user1 failed--test FAILED");
    }

    # read from user2
    $testfile = "test2.dat";
    $tmpfile = "/tmp/$testfile";
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/user2 -U user2 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from user2 succeeded--test PASSED");
    } else {
        log_message("user2 smbclient read from user2 failed--test FAILED");
    }
    
    # write to user2
    $ENV{"PASSWD"} = "passwd2";
    $rc = system_logged("smbclient //$server/user2 -U user2 -c 'put $tmpfile $testfile'");
    if ($rc == 0) {
        log_message("user2 smbclient write to user2 succeeded--test PASSED");
    } else {
        log_message("user2 smbclient write to user2 failed--test FAILED");
    }

    # read from user2 as user1
    $testfile = "test2.dat";
    $tmpfile = "/tmp/$testfile";
    $ENV{"PASSWD"} = "passwd1";
    $rc = system_logged("smbclient //$server/user2 -U user1 -c 'get $testfile $tmpfile'");
    if ($rc == 0) {
        log_message("user2 smbclient read from user2 as user1 succeeded--test FAILED");
    } else {
        log_message("user2 smbclient read from user2 as user1 failed--test PASSED");
    }
}

1;
